"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.buildDevStandalone = buildDevStandalone;
var _coreCommon = require("@storybook/core-common");
var _prompts = _interopRequireDefault(require("prompts"));
var _global = require("@storybook/global");
var _telemetry = require("@storybook/telemetry");
var _path = require("path");
var _nodeLogger = require("@storybook/node-logger");
var _devServer = require("./dev-server");
var _releaseNotes = require("./utils/release-notes");
var _outputStats = require("./utils/output-stats");
var _outputStartupInformation = require("./utils/output-startup-information");
var _updateCheck = require("./utils/update-check");
var _serverAddress = require("./utils/server-address");
var _getBuilders = require("./utils/get-builders");
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }
function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); enumerableOnly && (symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; })), keys.push.apply(keys, symbols); } return keys; }
function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = null != arguments[i] ? arguments[i] : {}; i % 2 ? ownKeys(Object(source), !0).forEach(function (key) { _defineProperty(target, key, source[key]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)) : ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } return target; }
function _defineProperty(obj, key, value) { key = _toPropertyKey(key); if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }
function _toPropertyKey(arg) { var key = _toPrimitive(arg, "string"); return typeof key === "symbol" ? key : String(key); }
function _toPrimitive(input, hint) { if (typeof input !== "object" || input === null) return input; var prim = input[Symbol.toPrimitive]; if (prim !== undefined) { var res = prim.call(input, hint || "default"); if (typeof res !== "object") return res; throw new TypeError("@@toPrimitive must return a primitive value."); } return (hint === "string" ? String : Number)(input); }
function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _unsupportedIterableToArray(arr, i) || _nonIterableRest(); }
function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }
function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }
function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) arr2[i] = arr[i]; return arr2; }
function _iterableToArrayLimit(arr, i) { var _i = null == arr ? null : "undefined" != typeof Symbol && arr[Symbol.iterator] || arr["@@iterator"]; if (null != _i) { var _s, _e, _x, _r, _arr = [], _n = !0, _d = !1; try { if (_x = (_i = _i.call(arr)).next, 0 === i) { if (Object(_i) !== _i) return; _n = !1; } else for (; !(_n = (_s = _x.call(_i)).done) && (_arr.push(_s.value), _arr.length !== i); _n = !0); } catch (err) { _d = !0, _e = err; } finally { try { if (!_n && null != _i.return && (_r = _i.return(), Object(_r) !== _r)) return; } finally { if (_d) throw _e; } } return _arr; } }
function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }
async function buildDevStandalone(options) {
  var packageJson = options.packageJson,
    versionUpdates = options.versionUpdates,
    releaseNotes = options.releaseNotes;
  var version = packageJson.version;

  // updateInfo and releaseNotesData are cached, so this is typically pretty fast
  var _await$Promise$all = await Promise.all([(0, _serverAddress.getServerPort)(options.port), versionUpdates ? (0, _updateCheck.updateCheck)(version) : Promise.resolve({
      success: false,
      cached: false,
      data: {},
      time: Date.now()
    }), releaseNotes ? (0, _releaseNotes.getReleaseNotesData)(version, _coreCommon.cache) : Promise.resolve((0, _releaseNotes.getReleaseNotesFailedState)(version))]),
    _await$Promise$all2 = _slicedToArray(_await$Promise$all, 3),
    port = _await$Promise$all2[0],
    versionCheck = _await$Promise$all2[1],
    releaseNotesData = _await$Promise$all2[2];
  if (!options.ci && !options.smokeTest && options.port != null && port !== options.port) {
    var _await$prompts = await (0, _prompts.default)({
        type: 'confirm',
        initial: true,
        name: 'shouldChangePort',
        message: `Port ${options.port} is not available. Would you like to run Storybook on port ${port} instead?`
      }),
      shouldChangePort = _await$prompts.shouldChangePort;
    if (!shouldChangePort) process.exit(1);
  }

  /* eslint-disable no-param-reassign */
  options.port = port;
  options.versionCheck = versionCheck;
  options.releaseNotesData = releaseNotesData;
  options.configType = 'DEVELOPMENT';
  options.configDir = (0, _path.resolve)(options.configDir);
  options.outputDir = options.smokeTest ? (0, _coreCommon.resolvePathInStorybookCache)('public') : (0, _path.resolve)(options.outputDir || (0, _coreCommon.resolvePathInStorybookCache)('public'));
  options.serverChannelUrl = (0, _serverAddress.getServerChannelUrl)(port, options);
  /* eslint-enable no-param-reassign */

  var _loadMainConfig = (0, _coreCommon.loadMainConfig)(options),
    framework = _loadMainConfig.framework;
  var corePresets = [];
  var frameworkName = typeof framework === 'string' ? framework : framework === null || framework === void 0 ? void 0 : framework.name;
  (0, _coreCommon.validateFrameworkName)(frameworkName);
  if (frameworkName) {
    corePresets.push((0, _path.join)(frameworkName, 'preset'));
  } else {
    _nodeLogger.logger.warn(`you have not specified a framework in your ${options.configDir}/main.js`);
  }

  // Load first pass: We need to determine the builder
  // We need to do this because builders might introduce 'overridePresets' which we need to take into account
  // We hope to remove this in SB8
  var presets = await (0, _coreCommon.loadAllPresets)(_objectSpread({
    corePresets: corePresets,
    overridePresets: []
  }, options));
  var _await$presets$apply = await presets.apply('core', undefined),
    renderer = _await$presets$apply.renderer,
    builder = _await$presets$apply.builder,
    disableTelemetry = _await$presets$apply.disableTelemetry;
  if (!options.disableTelemetry && !disableTelemetry) {
    if (versionCheck.success && !versionCheck.cached) {
      (0, _telemetry.telemetry)('version-update');
    }
  }
  var builderName = typeof builder === 'string' ? builder : builder === null || builder === void 0 ? void 0 : builder.name;
  var _await$Promise$all3 = await Promise.all([(0, _getBuilders.getPreviewBuilder)(builderName, options.configDir), (0, _getBuilders.getManagerBuilder)()]),
    _await$Promise$all4 = _slicedToArray(_await$Promise$all3, 2),
    previewBuilder = _await$Promise$all4[0],
    managerBuilder = _await$Promise$all4[1];

  // Load second pass: all presets are applied in order
  presets = await (0, _coreCommon.loadAllPresets)(_objectSpread({
    corePresets: [require.resolve('./presets/common-preset'), ...(managerBuilder.corePresets || []), ...(previewBuilder.corePresets || []), ...(renderer ? [(0, _coreCommon.resolveAddonName)(options.configDir, renderer, options)] : []), ...corePresets, require.resolve('./presets/babel-cache-preset')],
    overridePresets: previewBuilder.overridePresets
  }, options));
  var features = await presets.apply('features');
  _global.global.FEATURES = features;
  var fullOptions = _objectSpread(_objectSpread({}, options), {}, {
    presets: presets,
    features: features
  });
  var _await$storybookDevSe = await (0, _devServer.storybookDevServer)(fullOptions),
    address = _await$storybookDevSe.address,
    networkAddress = _await$storybookDevSe.networkAddress,
    managerResult = _await$storybookDevSe.managerResult,
    previewResult = _await$storybookDevSe.previewResult;
  var previewTotalTime = previewResult && previewResult.totalTime;
  var managerTotalTime = managerResult && managerResult.totalTime;
  var previewStats = previewResult && previewResult.stats;
  var managerStats = managerResult && managerResult.stats;
  if (options.webpackStatsJson) {
    var target = options.webpackStatsJson === true ? options.outputDir : options.webpackStatsJson;
    await (0, _outputStats.outputStats)(target, previewStats);
  }
  if (options.smokeTest) {
    var _managerStats$toJson, _previewStats$toJson;
    var warnings = [];
    warnings.push(...((managerStats === null || managerStats === void 0 ? void 0 : (_managerStats$toJson = managerStats.toJson()) === null || _managerStats$toJson === void 0 ? void 0 : _managerStats$toJson.warnings) || []));
    warnings.push(...((previewStats === null || previewStats === void 0 ? void 0 : (_previewStats$toJson = previewStats.toJson()) === null || _previewStats$toJson === void 0 ? void 0 : _previewStats$toJson.warnings) || []));
    var problems = warnings.filter(function (warning) {
      return !warning.message.includes(`export 'useInsertionEffect'`);
    }).filter(function (warning) {
      return !warning.message.includes(`compilation but it's unused`);
    }).filter(function (warning) {
      return !warning.message.includes(`Conflicting values for 'process.env.NODE_ENV'`);
    });

    // eslint-disable-next-line no-console
    console.log(problems.map(function (p) {
      return p.stack;
    }));
    process.exit(problems.length > 0 ? 1 : 0);
    return;
  }
  var name = frameworkName.split('@storybook/').length > 1 ? frameworkName.split('@storybook/')[1] : frameworkName;
  (0, _outputStartupInformation.outputStartupInformation)({
    updateInfo: versionCheck,
    version: version,
    name: name,
    address: address,
    networkAddress: networkAddress,
    managerTotalTime: managerTotalTime,
    previewTotalTime: previewTotalTime
  });
}