"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.storybookDevServer = storybookDevServer;
var _express = _interopRequireDefault(require("express"));
var _compression = _interopRequireDefault(require("compression"));
var _coreCommon = require("@storybook/core-common");
var _middleware = require("./utils/middleware");
var _serverAddress = require("./utils/server-address");
var _serverInit = require("./utils/server-init");
var _serverStatics = require("./utils/server-statics");
var _getServerChannel = require("./utils/get-server-channel");
var _openInBrowser = require("./utils/open-in-browser");
var _getBuilders = require("./utils/get-builders");
var _getStoryIndexGenerator = require("./utils/getStoryIndexGenerator");
var _doTelemetry = require("./utils/doTelemetry");
var _router = require("./utils/router");
var _getAccessControlMiddleware = require("./utils/getAccessControlMiddleware");
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }
function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _unsupportedIterableToArray(arr, i) || _nonIterableRest(); }
function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }
function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }
function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) arr2[i] = arr[i]; return arr2; }
function _iterableToArrayLimit(arr, i) { var _i = null == arr ? null : "undefined" != typeof Symbol && arr[Symbol.iterator] || arr["@@iterator"]; if (null != _i) { var _s, _e, _x, _r, _arr = [], _n = !0, _d = !1; try { if (_x = (_i = _i.call(arr)).next, 0 === i) { if (Object(_i) !== _i) return; _n = !1; } else for (; !(_n = (_s = _x.call(_i)).done) && (_arr.push(_s.value), _arr.length !== i); _n = !0); } catch (err) { _d = !0, _e = err; } finally { try { if (!_n && null != _i.return && (_r = _i.return(), Object(_r) !== _r)) return; } finally { if (_d) throw _e; } } return _arr; } }
function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }
async function storybookDevServer(options) {
  var _core$builder;
  var app = (0, _express.default)();
  var _await$Promise$all = await Promise.all([(0, _serverInit.getServer)(app, options), options.presets.apply('features'), options.presets.apply('core')]),
    _await$Promise$all2 = _slicedToArray(_await$Promise$all, 3),
    server = _await$Promise$all2[0],
    features = _await$Promise$all2[1],
    core = _await$Promise$all2[2];
  var serverChannel = (0, _getServerChannel.getServerChannel)(server);

  // try get index generator, if failed, send telemetry without storyCount, then rethrow the error
  var initializedStoryIndexGenerator = (0, _getStoryIndexGenerator.getStoryIndexGenerator)(features, options, serverChannel);
  (0, _doTelemetry.doTelemetry)(core, initializedStoryIndexGenerator, options);
  app.use((0, _compression.default)({
    level: 1
  }));
  if (typeof options.extendServer === 'function') {
    options.extendServer(server);
  }
  app.use((0, _getAccessControlMiddleware.getAccessControlMiddleware)(core === null || core === void 0 ? void 0 : core.crossOriginIsolated));

  // User's own static files
  var usingStatics = (0, _serverStatics.useStatics)(_router.router, options);
  (0, _middleware.getMiddleware)(options.configDir)(_router.router);
  app.use(_router.router);
  var port = options.port,
    host = options.host;
  var proto = options.https ? 'https' : 'http';
  var _getServerAddresses = (0, _serverAddress.getServerAddresses)(port, host, proto),
    address = _getServerAddresses.address,
    networkAddress = _getServerAddresses.networkAddress;
  var listening = new Promise(function (resolve, reject) {
    // @ts-expect-error (Following line doesn't match TypeScript signature at all 🤔)
    server.listen({
      port: port,
      host: host
    }, function (error) {
      return error ? reject(error) : resolve();
    });
  });
  var builderName = typeof (core === null || core === void 0 ? void 0 : core.builder) === 'string' ? core.builder : core === null || core === void 0 ? void 0 : (_core$builder = core.builder) === null || _core$builder === void 0 ? void 0 : _core$builder.name;
  var _await$Promise$all3 = await Promise.all([(0, _getBuilders.getPreviewBuilder)(builderName, options.configDir), (0, _getBuilders.getManagerBuilder)()]),
    _await$Promise$all4 = _slicedToArray(_await$Promise$all3, 2),
    previewBuilder = _await$Promise$all4[0],
    managerBuilder = _await$Promise$all4[1];
  if (options.debugWebpack) {
    (0, _coreCommon.logConfig)('Preview webpack config', await previewBuilder.getConfig(options));
  }
  var managerResult = await managerBuilder.start({
    startTime: process.hrtime(),
    options: options,
    router: _router.router,
    server: server,
    channel: serverChannel
  });
  var previewStarted = Promise.resolve();
  if (!options.ignorePreview) {
    previewStarted = previewBuilder.start({
      startTime: process.hrtime(),
      options: options,
      router: _router.router,
      server: server,
      channel: serverChannel
    }).catch(async function (e) {
      await (managerBuilder === null || managerBuilder === void 0 ? void 0 : managerBuilder.bail().catch());
      // For some reason, even when Webpack fails e.g. wrong main.js config,
      // the preview may continue to print to stdout, which can affect output
      // when we catch this error and process those errors (e.g. telemetry)
      // gets overwritten by preview progress output. Therefore, we should bail the preview too.
      await (previewBuilder === null || previewBuilder === void 0 ? void 0 : previewBuilder.bail().catch());

      // re-throw the error
      throw e;
    });
  }

  // this is a preview route, the builder has to be started before we can serve it
  // this handler keeps request to that route pending until the builder is ready to serve it, preventing a 404
  _router.router.get('/iframe.html', function (req, res, next) {
    // We need to catch here or node will treat any errors thrown by `previewStarted` as
    // unhandled and exit (even though they are very much handled below)
    previewStarted.catch(function () {}).then(function () {
      return next();
    });
  });
  Promise.all([initializedStoryIndexGenerator, listening, usingStatics]).then(async function () {
    if (!options.ci && !options.smokeTest && options.open) {
      (0, _openInBrowser.openInBrowser)(host ? networkAddress : address);
    }
  });
  return {
    previewResult: await previewStarted,
    managerResult: managerResult,
    address: address,
    networkAddress: networkAddress
  };
}