"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.withTelemetry = withTelemetry;
var _prompts = _interopRequireDefault(require("prompts"));
var _coreCommon = require("@storybook/core-common");
var _telemetry = require("@storybook/telemetry");
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }
function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); enumerableOnly && (symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; })), keys.push.apply(keys, symbols); } return keys; }
function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = null != arguments[i] ? arguments[i] : {}; i % 2 ? ownKeys(Object(source), !0).forEach(function (key) { _defineProperty(target, key, source[key]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)) : ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } return target; }
function _defineProperty(obj, key, value) { key = _toPropertyKey(key); if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }
function _toPropertyKey(arg) { var key = _toPrimitive(arg, "string"); return typeof key === "symbol" ? key : String(key); }
function _toPrimitive(input, hint) { if (typeof input !== "object" || input === null) return input; var prim = input[Symbol.toPrimitive]; if (prim !== undefined) { var res = prim.call(input, hint || "default"); if (typeof res !== "object") return res; throw new TypeError("@@toPrimitive must return a primitive value."); } return (hint === "string" ? String : Number)(input); }
var promptCrashReports = async function () {
  if (process.env.CI && process.env.NODE_ENV !== 'test') {
    return undefined;
  }
  var _await$prompts = await (0, _prompts.default)({
      type: 'confirm',
      name: 'enableCrashReports',
      message: `Would you like to send crash reports to Storybook?`,
      initial: true
    }),
    enableCrashReports = _await$prompts.enableCrashReports;
  await _coreCommon.cache.set('enableCrashReports', enableCrashReports);
  return enableCrashReports;
};
async function getErrorLevel({
  cliOptions: cliOptions,
  presetOptions: presetOptions
}) {
  if (cliOptions.disableTelemetry) return 'none';

  // If we are running init or similar, we just have to go with true here
  if (!presetOptions) return 'full';

  // should we load the preset?
  var presets = await (0, _coreCommon.loadAllPresets)(_objectSpread({
    corePresets: [require.resolve('./presets/common-preset')],
    overridePresets: []
  }, presetOptions));

  // If the user has chosen to enable/disable crash reports in main.js
  // or disabled telemetry, we can return that
  var core = await presets.apply('core');
  if ((core === null || core === void 0 ? void 0 : core.enableCrashReports) !== undefined) return core.enableCrashReports ? 'full' : 'error';
  if (core !== null && core !== void 0 && core.disableTelemetry) return 'none';

  // Deal with typo, remove in future version (7.1?)
  var valueFromCache = (await _coreCommon.cache.get('enableCrashReports')) ?? (await _coreCommon.cache.get('enableCrashreports'));
  if (valueFromCache !== undefined) return valueFromCache ? 'full' : 'error';
  var valueFromPrompt = await promptCrashReports();
  if (valueFromPrompt !== undefined) return valueFromPrompt ? 'full' : 'error';
  return 'full';
}
async function withTelemetry(eventType, options, run) {
  if (!options.cliOptions.disableTelemetry) (0, _telemetry.telemetry)('boot', {
    eventType: eventType
  }, {
    stripMetadata: true
  });
  try {
    await run();
  } catch (error) {
    try {
      var errorLevel = await getErrorLevel(options);
      if (errorLevel !== 'none') {
        var _options$presetOption;
        var precedingUpgrade = await (0, _telemetry.getPrecedingUpgrade)();
        await (0, _telemetry.telemetry)('error', {
          eventType: eventType,
          precedingUpgrade: precedingUpgrade,
          error: errorLevel === 'full' ? error : undefined
        }, {
          immediate: true,
          configDir: options.cliOptions.configDir || ((_options$presetOption = options.presetOptions) === null || _options$presetOption === void 0 ? void 0 : _options$presetOption.configDir),
          enableCrashReports: errorLevel === 'full'
        });
      }
    } catch (err) {
      // if this throws an error, we just move on
    }
    throw error;
  }
}